#!/bin/bash -x

source /etc/profile

##############################################################################
#This script controls the Real time proxy processing which includes:
#Preparing data sets for WRF/Chem run
#Running the WRF/Chem
#Preparing WRF/Chem output for the CRTM
#Running the CRTM
#Converting the CRTM output on the WRF/Chem grid to the final GRB files
#This script is set to run entirely on badger
#All pertinent data is stored at $cimss_disk
#All CRTM output is at $SAV_DISK/CRTMOUT

#The 8km WRF/Chem is initialized at 00z and run for 36 hours
#WRF/Chem is run with the GOCART aerosol package (chem_opt=300)
#CRTM data for all 16 ABI bands is output hourly from 12z day 1 to 11z day2
##############################################################################

#####set up computers and disks#########################################

export user_1=lenzen
export user_dir=/home/${user_1}
export TYPE=INTEL

#grid_dir must be the same as grid_dims set below under heading  WRF-CHEM information
export grid_dir=625X421

#path to program directories
export PROG_DIR=${user_dir}/PROJECTS/REALTIME_PROXY_ALL_BADGER_EAST/${grid_dir}

#export base_computer=badger
#export base_computer=s4-cardinal
export base_computer=
#export base_data_disk=/data/schaack
export base_data_disk=/data/users/${user_1}

#export computer_2=badger
#export computer_2=s4-cardinal
export computer_2=

#just used to rsync data to /cimss
#export computer_3=raqms-arc3
export computer_3=

#SAV_DISK - directory where all proxy results are written, first on scratch_d_wrf and then /cimss
export main_dir=REALTIME_PROXY_ALL_BADGER_EAST.3DAY.cldfix
#export cimss_disk=/cimss/SCHAACK/${main_dir}
#export cimss_disk=/proxy/lenzen/${main_dir}
#export cimss_disk=/cimss/lenzen/${main_dir}
export ccimss_disk=/cimss/lenzen/${main_dir}
export cimss_disk=/odyssey/proxy/lenzen/${main_dir}
#export cimss_disk=/iliad/proxy/lenzen/${main_dir}
#export cimss_disk_brdf=/proxy/lenzen/${main_dir}
export cimss_disk_brdf=/cimss/lenzen/${main_dir}

#CRTM Script directory
#export crtm_dir=/home/schaack/CRTM/REL-2.1/exec
#export crtm_script=/home/schaack/CRTM/REL-2.1/exec/SCRIPTS/SMP/${grid_dir}

#WRF-Chem directories
#export WRFCHEM_DIR=/data/schaack/WRF/WRFV3.5.1
export crtm_dir=/home/lenzen/CRTM/REL-2.1/exec
export crtm_script=/home/lenzen/CRTM/REL-2.1/exec/SCRIPTS/SMP/${grid_dir}

#WRF-Chem directories
export WRFCHEM_DIR=/data/users/lenzen/WRF/WRFV3.5.1
export WRFCHEM_DIR_WPS=/home/lenzen/WRF/WPSV
export WRFCHEM_DIR_WPS_BIN=$WRFCHEM_DIR/WPS/

#directory on base_computer where emissions and LBC progams are located
export WRFCHEM_UTILS=${base_data_disk}/WRFCHEM_UTILS

#FIRES DATA Location
#export FIRES_LOC=raqms-arc1:/data/jasonb/Fire_Files/goes-east
##export FIRES_LOC=raqms-arc1:/data/jasonb/Fire_Files/goes-east
# ajl change 3/19/2018 since goes-east goes-13 is gone and 16 is not ready yet
export FIRES_LOC=raqms-arc1:/data/jasonb/Fire_Files/goes-west

#RAQMS DATA
#export RAQMS_DIR=firefly:/sierra/CalNex/UW-Hybrid/Mission_1X1/Netcdf
export RAQMS_DIR=raqms-arc3:/data/CalNex/UW-Hybrid/Mission_1X1/Netcdf

#GFS_DATA
#export GFS_DATA=raqms-ops:/data/Met/GFS/Forecast/12Z/Original
export GFS_DATA=raqms-ops:/data/Met/GFS/Forecast/12Z/Original

#scratch directory on base_computer where get_initial is going to run, $base_data_disk/$get_init_dir
export get_init_dir=get_int_wps_badger_east_S4_3DAY.cldfix

#scratch directories used on badger
#scratch_d=/scratch1
scratch_d=/scratch
#scratch_d=/odyssey/scratch
export scratch_d_wps=${scratch_d}/${user_1}/$get_init_dir.$TYPE
export scratch_d_raqms=${scratch_d}/${user_1}/raqms_data
export scratch_d_wrf=${scratch_d}/${user_1}/realtime_badger_east_S4_3DAY.cldfix.$TYPE
export scratch_d_crtm=${scratch_d}/${user_1}/realtime_crtm_east_S4_3DAY.cldfix.$TYPE

#location of anthropogenic emissions
#export ANTRHO_EMISS=/data/${user_1}/REALTIME_PROXY/WRFCHEMI_8KM/NSSL_VERTICAL_GRID/${grid_dims}
export ANTRHO_EMISS=/data/users/${user_1}/REALTIME_PROXY/WRFCHEMI_8KM/NSSL_VERTICAL_GRID/${grid_dims}

#redefine base_data_disk

export base_data_disk=$scratch_d_wps   

#####done set base computer##########################################################


#WRFDA routine used to increment time
new_date=${user_dir}/UTILS/da_advance_time.exe

####define variables for WRF/Chem and CRTM runs#######################################

#Number of cpus to use for badger WRF/Chem run
#export NSLOTS=240
#export NSLOTS=288
#export NSLOTS=80 ! 4 minutes for 1 hr
export NSLOTS=120
#export NSLOTS=112
#export NSLOTS=40
export NCPU=$NSLOTS

#CRTM,  number of hours of WRF/Chem - CRTM data that we are going to process (12z day 1 - 11z day 2)
export nhrs=24
export time_inc=1         #time resoluton of CRTM output

#WRF-CHEM information
export npt_we=625
export npt_sn=421
export nlvl=35
export ref_lat=36.0
export ref_lon=-98.0
export truelat1=30.0
export truelat2=60.0
export stand_lon=-98.
export geog_data_res=5m
export dx_grid=8000
export dy_grid=8000
export map_proj=1
export grid_dims=${npt_we}X${npt_sn}
export npt_we_m1=$((${npt_we}-1))
export npt_sn_m1=$((${npt_sn}-1))

####export timestep=20        #WRF time step (secs)
export timestep=30        #WRF time step (secs)
export chemdtt=1          #chemistry time step (min)
#export fcst_length=48
export fcst_length=72 # make 3 days instead of two

#crtm processing info
#number of cpus on badger node
num_cpus=48
nsects=12
export max_cols=$npt_we_m1
export nr_sects=35
export nch_lw=10
export nch_sw=6
export nzz=5
export nll=1

if [ "${grid_dir}" != "${grid_dims}" ]; then
   echo 'ERROR: grid_dir must be the same as grid_dims'
   echo 'find ajl'
   exit
fi

#forecast hr to start and stop processing wrfout files
#we are not using the first $del_start hours 

export del_start=12
export del_stop=$fcst_length

####pertinent dates for job###########################################################

#Job is submitted via crontab on                      
#WRF/Chem forecast start day is same as submit day    WRF_START
#GFS data directory date is submit day minus 1        GFS_DATE
#FIRES directory date is same as GFS date
#RAQMS data directory date is submit day minus 2      RAQMS_DATE

#Get yyyymmdd for day that run is submitted 

get_time=$(date +%C%y%m%d)                           #the current year, month, day 

##get_time=20140318                                     #set get_time to run a particular day
get_time=2018102812
echo 'start date ',$get_time >> ${PROG_DIR}/dateeast
date >> ${PROG_DIR}/dateeast

year=`echo "${get_time}" | cut -c1-4`
month=`echo "${get_time}" | cut -c5-6`
day=`echo "${get_time}" | cut -c7-8`

hr='12'                                               #start hour for CRTM; 12z to 11z (24hrs)
export crtm_start=${year}${month}${day}${hr}
export crtm_stop=$($new_date $crtm_start ${nhrs}h)    #crtm stop time

#set WRF/Chem start hour
hr='00'                                               #start hour for WRF/Chem

#Set WRF/Chem start time
export WRF_START=${year}_${month}_${day}_${hr}        #start time for WRF/Chem
lday=${year}${month}${day}${hr}                       

gdate=$($new_date $lday ${fcst_length}h)
combine_stop=${gdate}

#Get GFS_DATE (Directory date name of GFS data)
gdate=$($new_date $lday -1d)
year=`echo "${gdate}" | cut -c1-4`
month=`echo "${gdate}" | cut -c5-6`
day=`echo "${gdate}" | cut -c7-8`
#hr=`echo "${gdate}" | cut -c9-10`
export GFS_DATE=${year}_${month}_${day}

#get RAQMS date  (Direcotry date name for RAQMS data)
gdate=$($new_date $lday -2d)
year=`echo "${gdate}" | cut -c1-4`
month=`echo "${gdate}" | cut -c5-6`
day=`echo "${gdate}" | cut -c7-8`
export RAQMS_DATE=${year}_${month}_${day}

echo ' '
echo 'Submit day, WRF_START, GFS_DATE, RAQMS_DATE  ' $WRF_START $GFS_DATE $RAQMS_DATE

rm ${user_dir}/comb*

###########################################################################################
#                     Do not remove anything above this line
###########################################################################################

#set up directories on scratch

setup=true

if [ $setup == 'true' ]; then

cd $scratch_d
mkdir ${user_1}
cd $scratch_d/${user_1}

#set up scratch disks

mkdir $scratch_d_wps
mkdir $scratch_d_wps/WPS
mkdir $scratch_d_wps/8KM_INPUT
mkdir $scratch_d_wps/8KM_INPUT/${grid_dims}
mkdir $scratch_d_wps/CLOUD_PROPERTIES
mkdir $scratch_d_wps/CLOUD_PROPERTIES/${grid_dims}
mkdir $scratch_d_wps/WRFOUT
mkdir $scratch_d_wps/WRFOUT/${grid_dims}
mkdir $scratch_d_wps/WRFOUT/SECTIONS
mkdir $scratch_d_wps/WRFOUT/SECTIONS/${grid_dims}
rsync -rltvD $WRFCHEM_DIR_WPS_BIN/geogrid* $scratch_d_wps/WPS/
rsync -rltvD $WRFCHEM_DIR_WPS_BIN/metgrid* $scratch_d_wps/WPS/
rsync -rltvD $PROG_DIR/WPS/geo*nc $scratch_d_wps/WPS/

mkdir $scratch_d_wrf
cd $scratch_d_wrf
mkdir WFABBA_fire_detection_proxy 
mkdir 8KM_INPUT
mkdir 8KM_INPUT/${grid_dims}
mkdir BRDF_DATA
mkdir BRDF_DATA/${grid_dims}
mkdir SFC_EMISS
mkdir SFC_EMISS/${grid_dims}
mkdir CLOUD_PROPERTIES
mkdir CLOUD_PROPERTIES/${grid_dims}
mkdir PARSED
mkdir PARSED/${grid_dims}
mkdir WRFOUT
mkdir WRFOUT/${grid_dims}
mkdir WRFOUT/${grid_dims}/SECTIONS
mkdir WRFOUT/${grid_dims}/SECTIONS/${grid_dims}
mkdir CRTMOUT
mkdir CRTMOUT/${grid_dims}
mkdir CRTMOUT/${grid_dims}/SECTIONS
mkdir CRTMOUT/${grid_dims}/FULL_GRID
mkdir CRTMOUT/${grid_dims}/Netcdf
mkdir CRTMOUT/${grid_dims}/Netcdf/Without_Time
mkdir CRTMOUT/${grid_dims}/Netcdf/Without_Time/Realtime
mkdir CRTMOUT/${grid_dims}/REMAP_2KM
mkdir CRTMOUT/${grid_dims}/GRB_FILES
mkdir UWHYB

mkdir $scratch_d_crtm
cd $scratch_d_crtm
mkdir $scratch_d_crtm/INPUT
mkdir $scratch_d_crtm/OUTPUT

mkdir $scratch_d_raqms
cd /data/users/lenzen/WRFCHEM_UTILS/WFABBA_fire_detection_proxy/
cp -p * $scratch_d_wrf/WFABBA_fire_detection_proxy/

cd ${PROG_DIR}

fi

echo '    '
echo '    '
echo '****Start Realtime Proxy CONUS GOES EAST**** '$(date)

####################rsync RAQMS data to scratch_d_raqms

#qsub -S /bin/bash -sync y -q serial.q -V get_raqms_data.sh
# already on serial node so why qsub again onto serial node ?
./get_raqms_data.3day.sh

###################Make WRF/Chem initial conditions, LBCs and emissions  

echo '****Start get_initial_conditions '$(date)

#qsub -S /bin/bash -l virtual_free=32G -sync y -q serial.q -V ${PROG_DIR}/get_initial_conditions_thomp_gen.sh 
/bin/rm -f jobfile.txt
sbatch --mem-per-cpu=32000 $BASENODETYPE -J GINT3DAY -t 60 ${PROG_DIR}/get_initial_conditions_thomp_gen.3day.intel.sh  >jobfile.txt.$TYPE
echo did get_initial
IDS=`gawk '{ print $4 }' jobfile.txt.$TYPE`

echo '****Done with get_initial **** '$(date)

###################RUn WRF-Chem  
echo '****Submit run_real.sh on badger**** '$(date)

#qsub -S /bin/bash -V -sync y -v PROG_DIR=$PROG_DIR -v grid_dims=$grid_dims -v scratch_d_wrf=$scratch_d_wrf -v identstart=$WRF_START -v raqms_date=$RAQMS_DATE -v user_1=${user_1} -v user_dir=${user_dir} -v main_dir=${main_dir} -v base_computer=${base_computer} -v base_data_disk=${base_data_disk} -v timestep=$timestep -v chemdtt=$chemdtt -v npt_we=$npt_we -v npt_sn=$npt_sn -v dx_grid=$dx_grid -v dy_grid=$dy_grid -v nlvl=$nlvl -v fcst_length=$fcst_length -pe mpi2_mpd $NSLOTS ${PROG_DIR}/run_real_thomp_v3.5.1.sh
/bin/rm -f jobfile.txt.$TYPE
# force it to wait for get initial job to finish with -d
echo 'ajl do sbatch'
#sbatch  -t 180 --nodelist=compute-205-[11,13,15,17,23] --exclusive -A star_amd -p amd -d afterany:$IDS -n $NSLOTS --export=PROG_DIR,grid_dims,scratch_d_wrf,identstart=$WRF_START,raqms_date=$RAQMS_DATE,user_1,user_dir,NSLOTS,main_dir,base_computer,WRFCHEM_DIR,base_data_disk,timestep,chemdtt,npt_we,npt_sn,dx_grid,dy_grid,nlvl,fcst_length,computer_2 ${PROG_DIR}/run_real_thomp_v3.5.1.sh >> jobfile.txt
sbatch  -t 20 --exclusive -p s4 -d afterany:$IDS -J IWRF3DAY -n $NSLOTS --export=PROG_DIR,ANTRHO_EMISS,grid_dims,scratch_d_wrf,identstart=$WRF_START,raqms_date=$RAQMS_DATE,user_1,user_dir,NSLOTS,main_dir,base_computer,WRFCHEM_DIR,base_data_disk,timestep,chemdtt,npt_we,npt_sn,dx_grid,dy_grid,nlvl,fcst_length,computer_2 ${PROG_DIR}/run_real_thomp_v3.5.1.3day.intel.sh >> jobfile.txt.$TYPE
# sbatch  -t 2 -A star_amd -p amd -d afterany:$IDS -n $NSLOTS --export=PROG_DIR,grid_dims,scratch_d_wrf,identstart=$WRF_START,raqms_date=$RAQMS_DATE,user_1,user_dir,NSLOTS,main_dir,base_computer,WRFCHEM_DIR,base_data_disk,timestep,chemdtt,npt_we,npt_sn,dx_grid,dy_grid,nlvl,fcst_length,computer_2 ${PROG_DIR}/run_real_thomp_v3.5.1.sh >> jobfile.txt
echo 'ajl did sbatch'
# ajl force it to wait for wrf to finish have it sleep for a minimum wrf run time to save sleeps
echo 'after sbatch for wrf run' >> ${PROG_DIR}/dateeast
date >> ${PROG_DIR}/dateeast
sbatch -J IRSYNWRF -t 360 -p serial -n 1 -d afterany:$IDS --export=scratch_d_wrf,grid_dims,cimss_disk,user_1,computer_3,WRF_START ${PROG_DIR}/run.rsync.wrfout.sh 

# sleep for 1 hour since if wrf run it will take 1.5 hours
$HOME/UTILS/sbatchwait jobfile.txt.$TYPE 60 2
sleep 60
# ajl wrf done by here just save wrfout
rsync -av ${scratch_d_wrf}/WRFOUT/${grid_dims}/$WRF_START ${ccimss_disk}/WRFOUT/${grid_dims}   
rsync -av ${scratch_d_wrf}/WRFOUT/${grid_dims}/$WRF_START ${ccimss_disk}/WRFOUT/${grid_dims}   
rsync -av ${scratch_d_wrf}/WRFOUT/${grid_dims}/$WRF_START ${cimss_disk}/WRFOUT/${grid_dims}   
rsync -av ${scratch_d_wrf}/WRFOUT/${grid_dims}/$WRF_START {cimss_disk}/WRFOUT/${grid_dims}   
rsync -av ${scratch_d_wrf}/$WRF_START ${cimss_disk}/WRF_INPUT/${grid_dims}
rsync -av ${scratch_d_wrf}/$WRF_START ${cimss_disk}/WRF_INPUT/${grid_dims}
exit 0
