#!/bin/bash 
ulimit -a >/home/lenzen/outulimit

echo 'run wrfchem, user_dir '${user_dir}
#WRFDA routine used to increment time
rsync -av ${user_dir}/UTILS/da_advance_time.exe .
new_date=./da_advance_time.exe

idstart=$ASSIM_START
idstop=$ASSIM_STOP
ident=${idstart}

year=`echo "${idstart}" | cut -c1-4`
month=`echo "${idstart}" | cut -c6-7`
day=`echo "${idstart}" | cut -c9-10`
hr=`echo "${idstart}" | cut -c12-13`
export metgrid_levels=32
if [ ${idstart} -lt 2016051112 ]; then
  export metgrid_levels=27
fi

stop_year=`echo "${idstop}" | cut -c1-4`
stop_month=`echo "${idstop}" | cut -c6-7`
stop_day=`echo "${idstop}" | cut -c9-10`
stop_hr=`echo "${idstop}" | cut -c12-13`

echo "Start WRF-Chem ,nslots " $NSLOTS
echo " "

stop_time=${stop_year}${stop_month}${stop_day}${stop_hr}

############################################################
############################################################
############################################################

current_time=${year}${month}${day}${hr}

#WHILE LOOP

while [ ${current_time} -lt ${stop_time} ] 
do

echo "*****Top of while loop in run_wrfchem_thomp.sh, start time, stop time ",${current_time} ${stop_time}
echo ' '

next_time=$($new_date $current_time ${fcst_length}h)

end_year=`echo "${next_time}" | cut -c1-4`
end_month=`echo "${next_time}" | cut -c5-6`
end_day=`echo "${next_time}" | cut -c7-8`
end_hr=`echo "${next_time}" | cut -c9-10`

cd ${Rundir}

run_days=0
run_hours=${fcst_length#0}
echo 'run_hours ',${run_hours}

rest_int=$((${run_hours}*60))

rest1=".false."

echo 'run forecast for ',${run_days},' days ',${run_hours},' hours'

###NAMELIST for forecast run
# ajl change wetscav_onoff flag 01/13/2017 ajl from 0 to -10 (special from george grill)
# ajl add lightning 1/23/2017

cat > namelist.input << __EOF
 &time_control
 run_days                            = ${run_days},
 run_hours                           = ${run_hours},
 run_minutes                         = 0,
 run_seconds                         = 0,
 start_year                          = ${year}, 2999, 2999,
 start_month                         = ${month},   06,   06,
 start_day                           = ${day},   11,   11,
 start_hour                          = ${hr},   12,   12,
 start_minute                        = 00,   00,   00,
 start_second                        = 00,   00,   00,
 end_year                            = ${end_year}, 2999, 2999,
 end_month                           = ${end_month},   06,   06,
 end_day                             = ${end_day},   12,   12,
 end_hour                            = ${end_hr},   12,   12,
 end_minute                          = 00,   00,   00,
 end_second                          = 00,   00,   00,
 interval_seconds                    = 21600,
 input_from_file                     = .true.,.true.,.true.,
 history_interval                    = 60,   60,   60,
 frames_per_outfile                  = 1, 1000, 1000,
 frames_per_auxinput7                = 1, 1000, 1000,
 frames_per_auxinput8                = 1, 1000, 1000,
 restart                             = .false.,
 restart_interval                    = 2529000,
 auxinput5_inname                    = wrfchemi_hhZ_d<domain>
 auxinput7_inname                    = wrffirechemi_d<domain>_<date>
 auxinput8_inname                    = wrfchemi_gocart_bg_d<domain>
 auxinput5_interval_m                = 60, 60, 60
 auxinput7_interval_m                = 4320, 60, 60
 auxinput8_interval_m                = 4320, 86400, 60
 io_form_history                     = 2
 io_form_restart                     = 2
 io_form_input                       = 2
 io_form_boundary                    = 2
 io_form_auxinput4                   = 2
 io_form_auxinput5                   = 2
 io_form_auxinput7                   = 2
 io_form_auxinput8                   = 2
 debug_level                         = 000
 auxinput1_inname                    = "met_em.d<domain>.<date>"
 /


 &domains
 time_step                           = ${timestep},
 time_step_fract_num                 = 0,
 time_step_fract_den                 = 1,
 max_dom                             = 1,
 e_we                                = ${npt_we},    112,   94,
 e_sn                                = ${npt_sn},    97,    91,
 e_vert                              = ${nlvl},    20,    20,
 dx                                  = ${dy_grid}, 20000, 6666.66,
 dy                                  = ${dy_grid}, 20000, 6666.66,
 p_top_requested                     = 5000,
 num_metgrid_levels                  = ${metgrid_levels},
 num_metgrid_soil_levels             = 4,
 grid_id                             = 1,     2,     3,
 parent_id                           = 1,     1,     2,
 i_parent_start                      = 1,     30,    30,
 j_parent_start                      = 1,     20,    30,
 parent_grid_ratio                   = 1,     3,     3,
 parent_time_step_ratio              = 1,     3,     3,
 feedback                            = 1,
 smooth_option                       = 0
 eta_levels                          =    1, 0.993, 0.983, 0.97, 0.954, 0.935, 0.913, 0.888, 0.86, 0.83, 0.799, 
    0.767, 0.733, 0.7, 0.667, 0.633, 0.6, 0.567, 0.533, 0.5, 0.467, 0.433, 
    0.4, 0.367, 0.333, 0.3, 0.267, 0.233, 0.2, 0.167, 0.133, 0.1, 0.067,0.033, 0.
 /


 &physics
 mp_physics                          = 8,     2,     2,
 lightning_option                    = 2,     2,     2,
 progn                               = 0,
 naer                                = 1.e9,
 ra_lw_physics                       = 1,     1,     1,
 ra_sw_physics                       = 1,     2,     2,
 radt                                = 8,    10,    10,
 sf_sfclay_physics                   = 2,     1,     1,
 sf_surface_physics                  = 2,     2,     2,
 bl_pbl_physics                      = 2,     1,     1,
 bldt                                = 0,     0,     0,
 cu_physics                          = 0,     5,     0,
 cudt                                = 0,     1,     1,
 isfflx                              = 1,
 ifsnow                              = 1,
 icloud                              = 1,
 surface_input_source                = 1,
 num_soil_layers                     = 4,
 sf_urban_physics                    = 0,
 swrad_scat                          = 1,
 mp_zero_out                         = 2,
 mp_zero_out_thresh                  = 1.e-8,
 maxiens                             = 1,
 maxens                              = 3,
 maxens2                             = 3,
 maxens3                             = 16,
 ensdim                              = 144,
 cu_rad_feedback                     = .false.,
 cu_diag                             = 1,
 do_radar_ref                        = 1,
 lightning_start_seconds             = 600.,
 lightning_dt                        = 30.,
 flashrate_factor                    = 1.,
 iccg_method                         = 2,
 cugd_avedx                          = 1
 /

 &fdda
 /

 &dynamics
 rk_ord                              = 3,
 w_damping                           = 1,
 diff_opt                            = 1,
 km_opt                              = 4,
 diff_6th_opt                        = 0,      0,      0,
 diff_6th_factor                     = 0.12,   0.12,   0.12,
 base_temp                           = 290.
 damp_opt                            = 0,
 zdamp                               = 5000.,  5000.,  5000.,
 dampcoef                            = 0.02,    0.2,    0.2
 khdif                               = 0,      0,      0,
 kvdif                               = 0,      0,      0,
 smdiv                               = 0.1,    0.1,    0.1,
 emdiv                               = 0.01,   0.01,   0.01,
 non_hydrostatic                     = .true., .true., .true.,
 tke_adv_opt                         = 2,      1,      1,
 moist_adv_opt                       = 2,      1,      1,
 scalar_adv_opt                      = 2,      1,      1,
 chem_adv_opt                        = 2,      1,      1,
 use_baseparam_fr_nml                = .true.,
 h_mom_adv_order                     = 5,      5,      5,
 v_mom_adv_order                     = 3,      3,      3,
 h_sca_adv_order                     = 5,      5,      5,
 v_sca_adv_order                     = 3,      3,      3,
 time_step_sound                     = 4,      4,      4,
 epssm                               = .1,     .1,     .1,
 /

 &bdy_control
 spec_bdy_width                      = 5,
 spec_zone                           = 1,
 relax_zone                          = 4,
 specified                           = .true., .false.,.false.,
 nested                              = .false., .true., .true.,
 /

 dampcoef                            = 0.01,    0.2,    0.2

 &grib2
 /


 &chem
 kemit                               = 19,
 chem_opt                            = 300,        2,
 bioemdt                             = 2160,       30,
 photdt                              = 2160,       30,
 chemdt                              = ${chemdtt},       2.,
 io_style_emissions                  = 1,
 emiss_inpt_opt                      = 1,        1,
 emiss_opt                           = 3,        3,
 chem_in_opt                         = 1,        0,
 phot_opt                            = 0,        1,
 gas_drydep_opt                      = 1,        1,
 aer_drydep_opt                      = 1,        1,
 aer_op_opt                          = 1,
 bio_emiss_opt                       = 0,        1,
 dust_opt                            = 1,
 dmsemis_opt                         = 0,
 seas_opt                            = 1,
 gas_bc_opt                          = 1,        1,
 gas_ic_opt                          = 1,        1,
 aer_bc_opt                          = 1,        1,
 aer_ic_opt                          = 1,        1,
 gaschem_onoff                       = 1,        1,
 aerchem_onoff                       = 1,        1,
 wetscav_onoff                       = -10,     -10,
 cldchem_onoff                       = 0,        0,
 vertmix_onoff                       = 1,        1,
 chem_conv_tr                        = 0,        1,
 chemdiag                            = 0,
 opt_pars_out                        = 1,
 biomass_burn_opt                    = 1,        0,
 plumerisefire_frq                   = 30,       30,
 aer_ra_feedback                     = 0,        0,
 have_bcs_chem                       = .true., .false.,
/

 &namelist_quilt
 nio_tasks_per_group = 1,
 nio_groups = 1,
 poll_servers = .false.,
 /

__EOF

# nio_tasks_per_group = 8,
##############################################################
##############################################################

#### Run wrf.exe for WRF-CHEM
cp -p namelist.input namelist.input.save3

echo 'Start running WRF/Chem '$(date)

echo 'qsub run forecast  '${year} ${month} ${day} ${hr}, ${end_year} ${end_month} ${end_day} ${end_hr}

srun --export=ALL -n $NSLOTS $Rundir/wrf.exe 


#check exit code to make sure run completed normally
if [ $? -ne 0 ]; then
echo 'Problem in WRF Chem'
# exit 111
fi

##############################################################
echo 'Stop wrf.exe '$(date)
##############################################################

year=${end_year}
month=${end_month}
day=${end_day}
hr=${end_hr}
current_time=${year}${month}${day}${hr}

echo
echo '#####################################'
echo 'Move data back to disk '

# Save output from real and wrf execution
#pwd

if [ ! -e ${ident} ]; then
 mkdir ${ident}
fi
if [ ! -e rsl_${ident} ]; then
 mkdir rsl_${ident}
fi
mv $Rundir/namelist.input  rsl_${ident}
mv $Rundir/rsl.out.*    rsl_${ident}
mv $Rundir/rsl.error.*  rsl_${ident}

mv $Rundir/wrfinput_d01    ${ident}
mv $Rundir/wrfbdy_d01      ${ident}
cp $Rundir/wrfchemi_gocart_bg_d01 ${ident}
echo ajl cp $Rundir/wrffirechemi*   ${ident}
cp $Rundir/wrffirechemi*   ${ident}

mkdir $Rundir/WRFOUT/${grid_dims}/${ident}
mv $Rundir/wrfout* $Rundir/WRFOUT/${grid_dims}/${ident}/

rm -f $Rundir/uwhyb*
rm -f $Rundir/met_em*

done
##############################################################
echo
echo 'end of time period ***************************************'
echo
echo
exit
